@@ -1,8 +1,10 @@
 package Imager::Matrix2d;
+use 5.006;
 use strict;
-use vars qw($VERSION);
+use Scalar::Util qw(reftype looks_like_number);
+use Carp qw(croak);
 
-$VERSION = "1.008";
+our $VERSION = "1.013";
 
 =head1 NAME
 
@@ -17,6 +19,9 @@ $VERSION = "1.008";
   $m4 = Imager::Matrix2d->shear(x=>$sx, y=>$sy);
   $m5 = Imager::Matrix2d->reflect(axis=>$axis);
   $m6 = Imager::Matrix2d->scale(x=>$xratio, y=>$yratio);
+  $m8 = Imager::Matric2d->matrix($v11, $v12, $v13,
+                                 $v21, $v22, $v23,
+                                 $v31, $v32, $v33);
   $m6 = $m1 * $m2;
   $m7 = $m1 + $m2;
   use Imager::Matrix2d qw(:handy);
@@ -26,7 +31,7 @@ $VERSION = "1.008";
 =head1 DESCRIPTION
 
 This class provides a simple wrapper around a reference to an array of
-9 co-efficients, treated as a matrix:
+9 coefficients, treated as a matrix:
 
  [ 0, 1, 2,
    3, 4, 5,
@@ -42,12 +47,11 @@ bottom, rotation angles are clockwise, rather than counter-clockwise.
 
 =cut
 
-use vars qw(@EXPORT_OK %EXPORT_TAGS @ISA);
-@ISA = 'Exporter';
-require 'Exporter.pm';
-@EXPORT_OK = qw(m2d_rotate m2d_identity m2d_translate m2d_shear 
+our @ISA = 'Exporter';
+require Exporter;
+our @EXPORT_OK = qw(m2d_rotate m2d_identity m2d_translate m2d_shear 
                 m2d_reflect m2d_scale);
-%EXPORT_TAGS =
+our %EXPORT_TAGS =
   (
    handy=> [ qw(m2d_rotate m2d_identity m2d_translate m2d_shear 
                 m2d_reflect m2d_scale) ],
@@ -56,7 +60,8 @@ require 'Exporter.pm';
 use overload 
   '*' => \&_mult,
   '+' => \&_add,
-  '""'=>\&_string;
+  '""'=>\&_string,
+  "eq" => \&_eq;
 
 =item identity()
 
@@ -97,9 +102,9 @@ sub rotate {
   if ($opts{'x'} || $opts{'y'}) {
     $opts{'x'} ||= 0;
     $opts{'y'} ||= 0;
-    return $class->translate('x'=>-$opts{'x'}, 'y'=>-$opts{'y'})
+    return $class->translate('x'=>$opts{'x'}, 'y'=>$opts{'y'})
       * $class->rotate(radians=>$angle)
-        * $class->translate('x'=>$opts{'x'}, 'y'=>$opts{'y'});
+        * $class->translate('x'=>-$opts{'x'}, 'y'=>-$opts{'y'});
   }
   else {
     my $sin = sin($angle);
@@ -207,7 +212,7 @@ sub reflect {
 
 Scales at the given ratios.
 
-You can also specify a center for the scaling with the cx and cy
+You can also specify a center for the scaling with the C<cx> and C<cy>
 parameters.
 
 =cut
@@ -235,6 +240,78 @@ sub scale {
   }
 }
 
+=item matrix($v11, $v12, $v13, $v21, $v22, $v23, $v31, $v32, $v33)
+
+Create a matrix with custom coefficients.
+
+=cut
+
+sub matrix {
+  my ($class, @self) = @_;
+
+  if (@self == 9) {
+    return bless \@self, $class;
+  }
+  else {
+    $Imager::ERRSTR = "9 coefficients required";
+    return;
+  }
+}
+
+=item transform($x, $y)
+
+Transform a point the same way matrix_transform does.
+
+=cut
+
+sub transform {
+  my ($self, $x, $y) = @_;
+
+  my $sz = $x * $self->[6] + $y * $self->[7] + $self->[8];
+  my ($sx, $sy);
+  if (abs($sz) > 0.000001) {
+    $sx = ($x * $self->[0] + $y * $self->[1] + $self->[2]) / $sz;
+    $sy = ($x * $self->[3] + $y * $self->[4] + $self->[5]) / $sz;
+  }
+  else {
+    $sx = $sy = 0;
+  }
+
+  return ($sx, $sy);
+}
+
+=item compose(matrix...)
+
+Compose several matrices together for use in transformation.
+
+For example, for three matrices:
+
+  my $out = Imager::Matrix2d->compose($m1, $m2, $m3);
+
+is equivalent to:
+
+  my $out = $m3 * $m2 * $m1;
+
+Returns the identity matrix if no parameters are supplied.
+
+May return the supplied matrix if only one matrix is supplied.
+
+=cut
+
+sub compose {
+  my ($class, @in) = @_;
+
+  @in
+    or return $class->identity;
+
+  my $out = pop @in;
+  for my $m (reverse @in) {
+    $out = $out * $m;
+  }
+
+  return $out;
+}
+
 =item _mult()
 
 Implements the overloaded '*' operator.  Internal use.
@@ -242,29 +319,53 @@ Implements the overloaded '*' operator.  Internal use.
 Currently both the left and right-hand sides of the operator must be
 an Imager::Matrix2d.
 
+When composing a matrix for transformation you should multiply the
+matrices in the reverse order of the transformations:
+
+  my $shear = Imager::Matrix2d->shear(x => 0.1);
+  my $rotate = Imager::Matrix2d->rotate(degrees => 45);
+  my $shear_then_rotate = $rotate * $shear;
+
+or use the compose method:
+
+  my $shear_then_rotate = Imager::Matrix2d->compose($shear, $rotate);
+
 =cut
+
 sub _mult {
   my ($left, $right, $order) = @_;
 
-  if (ref($right) && UNIVERSAL::isa($right, __PACKAGE__)) {
-    if ($order) {
-      ($left, $right) = ($right, $left);
-    }
-    my @result;
-    for my $i (0..2) {
-      for my $j (0..2) {
-        my $accum = 0;
-        for my $k (0..2) {
-          $accum += $left->[3*$i + $k] * $right->[3*$k + $j];
-        }
-        $result[3*$i+$j] = $accum;
+  if (ref($right)) {
+    if (reftype($right) eq "ARRAY") {
+      @$right == 9
+	or croak "9 elements required in array ref";
+      if ($order) {
+	($left, $right) = ($right, $left);
       }
+      my @result;
+      for my $i (0..2) {
+	for my $j (0..2) {
+	  my $accum = 0;
+	  for my $k (0..2) {
+	    $accum += $left->[3*$i + $k] * $right->[3*$k + $j];
+	  }
+	  $result[3*$i+$j] = $accum;
+	}
+      }
+      return bless \@result, __PACKAGE__;
     }
+    else {
+      croak "multiply by array ref or number";
+    }
+  }
+  elsif (defined $right && looks_like_number($right)) {
+    my @result = map $_ * $right, @$left;
+
     return bless \@result, __PACKAGE__;
   }
   else {
-    # presumably N * matrix or matrix * N
-    return undef; # for now
+    # something we don't handle
+    croak "multiply by array ref or number";
   }
 }
 
@@ -338,6 +439,21 @@ sub _string {
   $out;
 }
 
+=item _eq
+
+Implement the overloaded equality operator.
+
+Provided for older perls that don't handle magic auto generation of eq
+from "".
+
+=cut
+
+sub _eq {
+  my ($left, $right) = @_;
+
+  return $left . "" eq $right . "";
+}
+
 =back
 
 The following functions are shortcuts to the various constructors.
@@ -398,7 +514,7 @@ Tony Cook <tony@develop-help.com>
 
 =head1 BUGS
 
-Needs a way to invert matrixes.
+Needs a way to invert a matrix.
 
 =head1 SEE ALSO
 
