@@ -19,9 +19,10 @@ struct i_mutex_tag {
 };
 
 /*
-=item i_mutex_create()
+=item i_mutex_new()
 =category Mutex functions
-=synopsis i_mutex_t m = i_mutex_create();
+=synopsis i_mutex_t m = i_mutex_new();
+=order 10
 
 Create a mutex.
 
@@ -32,10 +33,12 @@ will abort.
 */
 
 i_mutex_t
-i_mutex_create(void) {
+i_mutex_new(void) {
   i_mutex_t m;
 
-  m = mymalloc(sizeof(*m));
+  m = malloc(sizeof(*m));
+  if (!m)
+    i_fatal(3, "Cannot allocate mutex object");
   InitializeCriticalSection(&(m->section));
 
   return m;
@@ -54,7 +57,7 @@ Destroy a mutex.
 void
 i_mutex_destroy(i_mutex_t m) {
   DeleteCriticalSection(&(m->section));
-  myfree(m);
+  free(m);
 }
 
 /*
