@@ -133,7 +133,7 @@ gnum(io_glue *ig, int *i) {
       return 0;
     }
     *i = work;
-    i_io_getc(ig);
+    i_io_nextc(ig);
   }
 
   return 1;
@@ -436,7 +436,6 @@ i_readpnm_wiol( io_glue *ig, int allow_incomplete) {
   i_img* im;
   int type;
   int width, height, maxval, channels;
-  int rounder;
   int c;
 
   i_clear_error();
@@ -528,7 +527,6 @@ i_readpnm_wiol( io_glue *ig, int allow_incomplete) {
       return NULL;
     }
   } else maxval=1;
-  rounder = maxval / 2;
 
   if ((c = i_io_getc(ig)) == EOF || !misspace(c)) {
     i_push_error(0, "garbage in header, invalid PNM file");
@@ -819,7 +817,7 @@ i_writeppm_wiol(i_img *im, io_glue *ig) {
       return(0);
     }
 
-    if (!im->virtual && im->bits == i_8_bits && im->type == i_direct_type
+    if (!i_img_virtual(im) && im->bits == i_8_bits && im->type == i_direct_type
 	&& im->channels == want_channels) {
       if (i_io_write(ig,im->idata,im->bytes) != im->bytes) {
         i_push_error(errno, "could not write ppm data");
