@@ -53,12 +53,13 @@ enum rm_byte_codes {
   rbc_jumpnz, /* jump if ra != 0 to jb */
   rbc_set, /* ra -> r */
   rbc_setp, /* pa -> p*/
-  rbc_print, /* prints ra */
+  rbc_print, /* print(ra) -> r -- prints, leaves on stack */
   rbc_rgba, /* rgba(ra, rb, rc, rd) -> p */
   rbc_hsva, /* hsva(ra, rb, rc, rd) -> p */
   rbc_alpha, /* alpha(pa) -> r */
   rbc_log, /* log(ra) -> r */
   rbc_exp, /* exp(ra) -> r */
+  rbc_det, /* det(ra, rb, rc, rd) -> r */
   rbc_op_count
 };
 
